ConRO.Hunter = {};
ConRO.Hunter.CheckTalents = function()
end
local ConRO_Hunter, ids = ...;

function ConRO:EnableRotationModule(mode)
	mode = mode or 1;
	self.ModuleOnEnable = ConRO.Hunter.CheckTalents;
	if mode == 1 then
		self.Description = 'Hunter [Beast Mastery - Ranged]';
		self.NextSpell = ConRO.Hunter.BeastMastery;
		self.ToggleDamage();
	end;
	if mode == 2 then
		self.Description = 'Hunter [Marksmanship - Ranged]';
		self.NextSpell = ConRO.Hunter.Marksmanship;
		self.ToggleDamage();
	end;
	if mode == 3 then
		self.Description = 'Hunter [Survival - Melee]';
		self.NextSpell = ConRO.Hunter.Survival;
		self.ToggleDamage();
	end;
	self:RegisterEvent('UNIT_SPELLCAST_SUCCEEDED');
	self.lastSpellId = 0;
end

function ConRO:EnableDefenseModule(mode)
	mode = mode or 1;
	self.ModuleOnEnable = ConRO.Hunter.CheckTalents;
	if mode == 1 then
		self.NextDef = ConRO.Hunter.BeastMasteryDef;
	end;
	if mode == 2 then
		self.NextDef = ConRO.Hunter.MarksmanshipDef;
	end;
	if mode == 3 then
		self.NextDef = ConRO.Hunter.SurvivalDef;
	end;
end

function ConRO:UNIT_SPELLCAST_SUCCEEDED(event, unitID, lineID, spellID)
	if unitID == 'player' then
		self.lastSpellId = spellID;
	end
end

function ConRO.Hunter.BeastMastery(_, timeShift, currentSpell, gcd, tChosen)
--Resources
	local focus 											= UnitPower('player', Enum.PowerType.Focus);
	local focusMax 											= UnitPowerMax('player', Enum.PowerType.Focus);
	local raidHaste, raidSated 								= ConRO:Heroism();

--Abilities	
	local kc, kcCD	 										= ConRO:AbilityReady(ids.BM_Ability.KillCommand, timeShift);
	local bw 												= ConRO:AbilityReady(ids.BM_Ability.BestialWrath, timeShift);
		local bwBuff 											= ConRO:Aura(ids.BM_Buff.BestialWrath, timeShift);
	local aotw 												= ConRO:AbilityReady(ids.BM_Ability.AspectoftheWild, timeShift);
	local cshot 											= ConRO:AbilityReady(ids.BM_Ability.CounterShot, timeShift);
	local multi 											= ConRO:AbilityReady(ids.BM_Ability.MultiShot, timeShift);
		local bcBuff, _, bcBuffDur 								= ConRO:Aura(ids.BM_Buff.BeastCleave, timeShift + 1);
	local cobra 											= ConRO:AbilityReady(ids.BM_Ability.CobraShot, timeShift);
	local bshot, bsCD										= ConRO:AbilityReady(ids.BM_Ability.BarbedShot, timeShift);
		local bsCharges, _, bsCCD, bsMCCD						= ConRO:SpellCharges(ids.BM_Ability.BarbedShot);
		local frenzyBuff, frenzyCount, frenzyDUR				= ConRO:UnitAura(ids.BM_Buff.Frenzy, timeShift, 'pet');
	local prage 											= ConRO:AbilityReady(ids.BM_Ability.PrimalRage, timeShift);	
	
	local db, dbCD	 										= ConRO:AbilityReady(ids.BM_Talent.DireBeast, timeShift);		
	local stamp 											= ConRO:AbilityReady(ids.BM_Talent.Stampede, timeShift);	
	local amoc 												= ConRO:AbilityReady(ids.BM_Talent.AMurderofCrows, timeShift);	
	local chimaera 											= ConRO:AbilityReady(ids.BM_Talent.ChimaeraShot, timeShift);
	local barrage 											= ConRO:AbilityReady(ids.BM_Talent.Barrage, timeShift);
	local scobra 											= ConRO:AbilityReady(ids.BM_Talent.SpittingCobra, timeShift);
	
--Conditions	
	local targetPh 											= ConRO:PercentHealth('target');
	local summoned 											= ConRO:CallPet();
	local assist 											= ConRO:PetAssist();
	
--Indicators
	ConRO:AbilityInterrupt(ids.BM_Ability.CounterShot, cshot and ConRO:Interrupt());

	ConRO:AbilityBurst(ids.BM_Talent.Stampede, stamp);
	ConRO:AbilityBurst(ids.BM_Ability.AspectoftheWild, bwBuff and aotw and ConRO_BurstButton:IsVisible());
	ConRO:AbilityBurst(ids.BM_Ability.BestialWrath, bw and ConRO_BurstButton:IsVisible());
	
--Warnings
	if not summoned then
		UIErrorsFrame:AddMessage("Call your pet!!!", 1.0, 0.0, 0.0, 10);
	end
	
--Rotations
	if (bshot or bsCD <= ConRO:GlobalCooldown()) and (bsCharges == 2 or (bsCharges == 1 and bsCCD <= 1.5) or (frenzyBuff and frenzyDUR < 1.5)) then
		return ids.BM_Ability.BarbedShot;
	end

	if multi and not bcBuff and ConRO_AoEButton:IsVisible() then
		return ids.BM_Ability.MultiShot;
	end
	
	if barrage and ConRO_AoEButton:IsVisible() then
		return ids.BM_Talent.Barrage;
	end

	if kc then
		return ids.BM_Ability.KillCommand;
	end
	
	if chimaera then
		return ids.BM_Talent.ChimaeraShot;
	end		

	if amoc then
		return ids.BM_Talent.AMurderofCrows;
	end
	
	if aotw and bwBuff and ConRO_FullButton:IsVisible() then
		return ids.BM_Ability.AspectoftheWild;
	end	
	
	if bw and not bwBuff and ConRO_FullButton:IsVisible() then
		return ids.BM_Ability.BestialWrath;
	end		
	
	if db then
		return ids.BM_Talent.DireBeast;
	end	
	
	if cobra and kcCD >= 2.5 and ConRO_SingleButton:IsVisible() then
		return ids.BM_Ability.CobraShot;
	end
return ids.BM_Ability.KillCommand;
end

function ConRO.Hunter.BeastMasteryDef(_, timeShift, currentSpell, gcd, tChosen)
--Abilities
	local exhil												= ConRO:AbilityReady(ids.BM_Ability.Exhilaration, timeShift);
	local aott 												= ConRO:AbilityReady(ids.BM_Ability.AspectoftheTurtle, timeShift);
	local mendpet											= ConRO:AbilityReady(ids.BM_Ability.MendPet, timeShift);	
	local feedpet											= ConRO:AbilityReady(ids.BM_Ability.FeedPet, timeShift);

--Conditions	
	local playerPh 											= ConRO:PercentHealth('player');
	local petPh												= ConRO:PercentHealth('pet');
	local incombat 											= UnitAffectingCombat('player');
	local summoned 											= ConRO:CallPet();
	
--Rotations	
	if feedpet and not incombat and summoned and petPh <= 60 then
		return ids.BM_Ability.FeedPet;
	end	
	
	if exhil and (playerPh <= 50 or petPh <= 20) then
		return ids.BM_Ability.Exhilaration;
	end
	
	if mendpet and summoned and petPh <= 60 then
		return ids.BM_Ability.MendPet;
	end	
	
	if aott then
		return ids.BM_Ability.AspectoftheTurtle;
	end
	
	return nil;
end

function ConRO.Hunter.Marksmanship(_, timeShift, currentSpell, gcd, tChosen)
--Resources
	local focus 											= UnitPower('player', Enum.PowerType.Focus)
	local focusMax 											= UnitPowerMax('player', Enum.PowerType.Focus);

--Abilities	
	local cshot 											= ConRO:AbilityReady(ids.MM_Ability.CounterShot, timeShift);
	local tshot 											= ConRO:AbilityReady(ids.MM_Ability.Trueshot, timeShift);
	local as 												= ConRO:AbilityReady(ids.MM_Ability.ArcaneShot, timeShift);
	local mshot												= ConRO:AbilityReady(ids.MM_Ability.MultiShot, timeShift);
		local tsBuff											= ConRO:Aura(ids.MM_Buff.TrickShots, timeShift);
	local aimshot									 		= ConRO:AbilityReady(ids.MM_Ability.AimedShot, timeShift);
		local asCharges, _, asCCD, asMCCD						= ConRO:SpellCharges(ids.MM_Ability.AimedShot);		
		local psBuff											= ConRO:Aura(ids.MM_Buff.PreciseShots, timeShift);
	local rfire												= ConRO:AbilityReady(ids.MM_Ability.RapidFire, timeShift);
	local ss 												= ConRO:AbilityReady(ids.MM_Ability.SteadyShot, timeShift);
		local lsBuff											= ConRO:Aura(ids.MM_Buff.LethalShots, timeShift);

	local amoc 												= ConRO:AbilityReady(ids.MM_Talent.AMurderofCrows, timeShift);
	local pshot 											= ConRO:AbilityReady(ids.MM_Talent.PiercingShot, timeShift);
	local expshot 											= ConRO:AbilityReady(ids.MM_Talent.ExplosiveShot, timeShift);
	local barrage 											= ConRO:AbilityReady(ids.MM_Talent.Barrage, timeShift);
	local ssting											= ConRO:AbilityReady(ids.MM_Talent.SerpentSting, timeShift);
		local sfDebuff, sfCD									= ConRO:TargetAura(ids.MM_Debuff.SerpentSting, timeShift + 3);
	local hm												= ConRO:AbilityReady(ids.MM_Talent.HuntersMark, timeShift);		
		local hmDebuff 											= ConRO:PersistentDebuff(ids.MM_Debuff.HuntersMark);
	local dtap												= ConRO:AbilityReady(ids.MM_Talent.DoubleTap, timeShift);
		local dtBuff											= ConRO:Aura(ids.MM_Buff.DoubleTap, timeShift);
	
	local lol, lolCharges 									= ConRO:Aura(ids.MM_Buff.LockandLoad, timeShift);
	local ts, _, tsCD 										= ConRO:Aura(ids.MM_Buff.Trueshot, timeShift);
	local sf 												= ConRO:Aura(ids.MM_Buff.SteadyFocus, timeShift);

--Conditions	
	local targetPh 											= ConRO:PercentHealth('target');
	local summoned 											= ConRO:CallPet();
	local assist 											= ConRO:PetAssist();
	
	local _, _, _, aimedShotTimemil = GetSpellInfo(ids.MM_Ability.AimedShot);
	local aimedShotTime = aimedShotTimemil*.001;
	local aimedShotError = 0.3;

	if currentSpell == ids.MM_Ability.AimedShot then
		focus = focus - 30;
	end
	
	local rfFocusCutOff = 70
	
	if tChosen[ids.MM_Talent.Streamline] then
		rfFocusCutOff = 64;
	end
	
	if lol then
		aimedShotTime = gcd;
	end
	
	local expShotName = GetSpellInfo('Explosive Shot');
	

--Indicators	
	ConRO:AbilityInterrupt(ids.MM_Ability.CounterShot, cshot and ConRO:Interrupt());
	
	ConRO:AbilityBurst(ids.MM_Ability.Trueshot, tshot and asCharges == 0 and ConRO_BurstButton:IsVisible());
	ConRO:AbilityBurst(ids.MM_Talent.DoubleTap, dtap);
	
--Warnings	


--Rotations
	if ConRO.lastSpellId == ids.MM_Talent.ExplosiveShot and expShotName == 'Explosive Shot: Detonate!' then
		return ids.MM_Talent.ExplosiveShotDetonate;
	end	
	
	if hm and not hmDebuff then
		return ids.MM_Talent.HuntersMark;
	end
	
	if tshot and asCharges == 0 and ConRO_FullButton:IsVisible() then
		return ids.MM_Ability.Trueshot;
	end
	
	if rfire and (lsBuff or dtBuff) then
		if ConRO_SingleButton:IsVisible() then
			return ids.MM_Ability.RapidFire;
		elseif ConRO_AoEButton:IsVisible() and tsBuff then
			return ids.MM_Ability.RapidFire;
		end
	end

	if lol then
		if aimshot and not psBuff and (asCharges == 2 or (asCharges == 1 and asCCD <= aimedShotTime + .5)) and currentSpell ~= ids.MM_Ability.AimedShot then
			if ConRO_SingleButton:IsVisible() then
				return ids.MM_Ability.AimedShot;
			elseif ConRO_AoEButton:IsVisible() and tsBuff then
				return ids.MM_Ability.AimedShot;
			end
		end
	else
		if aimshot and (asCharges == 2 or (asCharges == 1 and asCCD <= aimedShotTime + .5)) and currentSpell ~= ids.MM_Ability.AimedShot then
			if ConRO_SingleButton:IsVisible() then
				return ids.MM_Ability.AimedShot;
			elseif ConRO_AoEButton:IsVisible() and tsBuff then
				return ids.MM_Ability.AimedShot;
			end
		end	
	end
	
	if ConRO_AoEButton:IsVisible() then
		if mshot and not tsBuff then
			return ids.MM_Ability.MultiShot;
		end	
	end

	if barrage and ConRO_AoEButton:IsVisible() then
		return ids.MM_Talent.Barrage;
	end
	
	if expshot and ConRO_AoEButton:IsVisible() then
		return ids.MM_Talent.ExplosiveShot;
	end
	
	if pshot then
		return ids.MM_Talent.PiercingShot;
	end
	
	if amoc then
		return ids.MM_Talent.AMurderofCrows;
	end
	
	if ssting and not ssDebuff then
		return ids.MM_Talent.SerpentSting;
	end
	
	if expshot and ConRO_SingleButton:IsVisible() then
		return ids.MM_Talent.ExplosiveShot;
	end
	
	if ConRO_AoEButton:IsVisible() then
		if mshot and (psBuff or currentSpell == ids.MM_Ability.AimedShot) then
			return ids.MM_Ability.MultiShot;
		end	
	else
		if as and (psBuff or currentSpell == ids.MM_Ability.AimedShot) then
			return ids.MM_Ability.ArcaneShot;
		end
	end
	
	if aimshot and currentSpell ~= ids.MM_Ability.AimedShot then
		return ids.MM_Ability.AimedShot;
	end	
	
	if ConRO_AoEButton:IsVisible() then
		if mshot and focus >= 90 then
			return ids.MM_Ability.MultiShot;
		end	
	else
		if as and focus >= 90 then
			return ids.MM_Ability.ArcaneShot;
		end
	end
	
	if ss then
		return ids.MM_Ability.SteadyShot;
	end
	return nil;
end

function ConRO.Hunter.MarksmanshipDef(_, timeShift, currentSpell, gcd, tChosen)
--Abilities	
	local exhil 											= ConRO:AbilityReady(ids.MM_Ability.Exhilaration, timeShift);
	local aott 												= ConRO:AbilityReady(ids.MM_Ability.AspectoftheTurtle, timeShift);
	local sotf												= ConRO:AbilityReady(ids.MM_Ability.SurvivaloftheFittestLW, timeShift);
		local lw												= ConRO:Form(ids.MM_Form.LoneWolf);
	local mendpet											= ConRO:AbilityReady(ids.MM_Ability.MendPet, timeShift);	
	local feedpet											= ConRO:AbilityReady(ids.MM_Ability.FeedPet, timeShift);

--Conditions	
	local playerPh 											= ConRO:PercentHealth('player');
	local petPh												= ConRO:PercentHealth('pet');
	local incombat 											= UnitAffectingCombat('player');
	local summoned 											= ConRO:CallPet();
	
--Rotations	
	if feedpet and summoned and not incombat and petPh <= 60 then
		return ids.MM_Ability.FeedPet;
	end	
	
	if exhil and (playerPh <= 50 or petPh <= 20) then
		return ids.MM_Ability.Exhilaration;
	end
	
	if mendpet and summoned and petPh <= 60 then
		return ids.MM_Ability.MendPet;
	end	
	
	if aott then
		return ids.MM_Ability.AspectoftheTurtle;
	end
	
	if sotf and lw and incombat then
		return ids.MM_Ability.SurvivaloftheFittestLW;
	end
	
	return nil;
end

function ConRO.Hunter.Survival(_, timeShift, currentSpell, gcd, tChosen)
--Resources	
	local focus 											= UnitPower('player', Enum.PowerType.Focus);
	local focusMax											= UnitPowerMax('player', Enum.PowerType.Focus);
	
--Abilities	
	local muz 												= ConRO:AbilityReady(ids.Surv_Ability.Muzzle, timeShift);
	local aote 												= ConRO:AbilityReady(ids.Surv_Ability.AspectoftheEagle, timeShift);
		local aoteBuff											= ConRO:Aura(ids.Surv_Buff.AspectoftheEagle, timeShift);
	local harpoon 											= ConRO:AbilityReady(ids.Surv_Ability.Harpoon, timeShift);
	local carve 											= ConRO:AbilityReady(ids.Surv_Ability.Carve, timeShift);
	local kc 												= ConRO:AbilityReady(ids.Surv_Ability.KillCommand, timeShift);
	local rstrike 											= ConRO:AbilityReady(ids.Surv_Ability.RaptorStrike, timeShift);
		local vvBuff 											= ConRO:Aura(ids.Surv_Buff.VipersVenom, timeShift);
	local ssting											= ConRO:AbilityReady(ids.Surv_Ability.SerpentSting, timeShift);
		local ssDebuff 											= ConRO:TargetAura(ids.Surv_Debuff.SerpentSting, timeShift + 3);
	local wfb 												= ConRO:AbilityReady(ids.Surv_Ability.WildfireBomb, timeShift);
		local wfbDebuff 										= ConRO:TargetAura(ids.Surv_Debuff.WildfireBomb, timeShift + 1);
		local sbDebuff											= ConRO:TargetAura(ids.Surv_Debuff.ShrapnelBomb, timeShift + 1);
		local vbDebuff											= ConRO:TargetAura(ids.Surv_Debuff.VolatileBomb, timeShift + 1);
		local pbDebuff											= ConRO:TargetAura(ids.Surv_Debuff.PheromoneBomb, timeShift + 1);
	local cassault											= ConRO:AbilityReady(ids.Surv_Ability.CoordinatedAssault, timeShift);
		
	local mb												= ConRO:AbilityReady(ids.Surv_Talent.MongooseBite, timeShift);
		local mfBuff, mfCount, mfCD								= ConRO:Aura(ids.Surv_Buff.MongooseFury, timeShift);	
	local fs 												= ConRO:AbilityReady(ids.Surv_Talent.FlankingStrike, timeShift);	
	local butch 											= ConRO:AbilityReady(ids.Surv_Talent.Butchery, timeShift);	
	local amoc 												= ConRO:AbilityReady(ids.Surv_Talent.AMurderofCrows, timeShift);
	local strap												= ConRO:AbilityReady(ids.Surv_Talent.SteelTrap, timeShift);
	local chakrams											= ConRO:AbilityReady(ids.Surv_Talent.Chakrams, timeShift);

--Conditions	
	local inRange 											= ConRO:IsSpellInRange(ids.Surv_Ability.Harpoon, 'target');
	local inMelee											= ConRO:IsSpellInRange(ids.Surv_Ability.WingClip, 'target');
	local incombat 											= UnitAffectingCombat('player');
	local summoned 											= ConRO:CallPet();
	local assist 											= ConRO:PetAssist();

	local wildfireBomb = ids.Surv_Ability.WildfireBomb;
	
	if ConRO:FindSpell(ids.Surv_Talent.ShrapnelBomb) then
		wildfireBomb = ids.Surv_Talent.ShrapnelBomb;
	elseif ConRO:FindSpell(ids.Surv_Talent.PheromoneBomb) then
		wildfireBomb = ids.Surv_Talent.PheromoneBomb;
	elseif ConRO:FindSpell(ids.Surv_Talent.VolatileBomb) then
		wildfireBomb = ids.Surv_Talent.VolatileBomb;
	end
	
	local raptorStrike = ids.Surv_Ability.RaptorStrike;
	local mongooseBite = ids.Surv_Talent.MongooseBite;
	
	if aoteBuff then
		raptorStrike = ids.Surv_Ability.RaptorStrikeRanged;
		mongooseBite = ids.Surv_Talent.MongooseBiteRanged;
	end
	
--Indicators	
	ConRO:AbilityInterrupt(ids.Surv_Ability.Muzzle, muz and ConRO:Interrupt());
	
	ConRO:AbilityBurst(ids.Surv_Ability.CoordinatedAssault, cassault and ConRO_BurstButton:IsVisible());
	ConRO:AbilityBurst(ids.Surv_Ability.AspectoftheEagle, aote and incombat and not inMelee);
	
--Warnings	
	if not summoned then
		UIErrorsFrame:AddMessage("Call your pet!!!", 1.0, 0.0, 0.0, 53, 5);
	end

--Rotations
	if cassault and ConRO_FullButton:IsVisible() then
		return ids.Surv_Ability.CoordinatedAssault;
	end
	
	if ssting and not ssDebuff and not incombat then
		return ids.Surv_Ability.SerpentSting;
	end
	
	if not inMelee and inRange and harpoon then
		return ids.Surv_Ability.Harpoon;
	end	
	
	if kc and focus <= 85 then
		return ids.Surv_Ability.KillCommand;
	end

	if ssting and vvBuff and mf then
		return ids.Surv_Ability.SerpentSting;
	end	
	
	if wfb and not wfbDebuff then
		return wildfireBomb;
	end
	
	if chakrams then
		return ids.Surv_Talent.Chakrams;
	end
	
	if strap then
		return ids.Surv_Talent.SteelTrap;
	end
	
	if amoc then
		return ids.Surv_Talent.AMurderofCrows;
	end
	
	if ConRO_AoEButton:IsVisible() then
		if tChosen[ids.Surv_Talent.Butchery] then
			if butch then
				return ids.Surv_Talent.Butchery;
			end
		else
			if carve then
				return ids.Surv_Ability.Carve;
			end
		end
	end	

	if ssting and (not ssDebuff or vvBuff) then
		return ids.Surv_Ability.SerpentSting;
	end	
	
	if harpoon and tChosen[ids.Surv_Talent.TermsofEngagement] then
		return ids.Surv_Ability.Harpoon;
	end

	if fs and focus >= 65 then
		return ids.Surv_Talent.FlankingStrike;
	end
	
	if tChosen[ids.Surv_Talent.MongooseBite] then
		if mb and ((not mf and focus >= 90) or (mfCD >= .5 and mfCount >= 1)) then
			return mongooseBite;
		end
	else
		if rstrike and focus >= 75 then
			return raptorStrike;
		end
	end
	return nil;
end

function ConRO.Hunter.SurvivalDef(_, timeShift, currentSpell, gcd, tChosen)
--Abilities
	local exhil 											= ConRO:AbilityReady(ids.Surv_Ability.Exhilaration, timeShift);
	local aott 												= ConRO:AbilityReady(ids.Surv_Ability.AspectoftheTurtle, timeShift);
	local mendpet											= ConRO:AbilityReady(ids.Surv_Ability.MendPet, timeShift);	
	local feedpet											= ConRO:AbilityReady(ids.Surv_Ability.FeedPet, timeShift);

--Conditions	
	local playerPh 											= ConRO:PercentHealth('player');
	local petPh												= ConRO:PercentHealth('pet');
	local incombat 											= UnitAffectingCombat('player');
	local summoned 											= ConRO:CallPet();
	
--Rotations	
	if feedpet and summoned and not incombat and petPh <= 60 then
		return ids.Surv_Ability.FeedPet;
	end	
	
	if exhil and (playerPh <= 50 or petPh <= 20) then
		return ids.Surv_Ability.Exhilaration;
	end
	
	if mendpet and summoned and petPh <= 60 then
		return ids.Surv_Ability.MendPet;
	end	
	
	if aott then
		return ids.Surv_Ability.AspectoftheTurtle;
	end
	
	return nil;
end